from uperations.operation import Operation
from abc import abstractmethod
import subprocess

class Dockeroperation(Operation):

    def _parser(self, main_parser):
        main_parser.add_argument('-d','--docker-container', dest='docker_container', help="URL of the docker container.")
        main_parser.add_argument('-n','--no-pull', dest='no_pull', help="Don't pull the docker container before running.")
        return main_parser

    def _before_start(self):
        super(Dockeroperation, self)._before_start()

        if not self.args.no_pull:
            print(self._docker_container())

            subprocess.check_output(['docker','pull',self._docker_container()])
        return True

    def _on_running(self):
        super(Dockeroperation, self)._on_running()
        return True

    def _on_error(self, exception):
        super(Dockeroperation, self)._on_error(exception)
        return True

    def _on_completed(self):
        super(Dockeroperation, self)._on_completed()
        return True

    def _run(self):
        subprocess.check_output(['docker','run',self._docker_container()] + self.unknown_args)

    @abstractmethod
    def _docker_container(self):
        raise NotImplementedError



from uperations.operation import Operation
from ..utils import library_create

class Makelibrary(Operation):

    def _schema(self):
        return {}

    @staticmethod
    def name():
        return "make_library"

    @staticmethod
    def description():
        return "Create a new library for operations"

    def _parser(self, main_parser):
        main_parser.add_argument('name')
        return

    def _run(self):
        library_create(self.args.name,'libraries')
        return True


from uperations.operation import Operation
from argparse import FileType
from dotenv import load_dotenv

class DotEnvOperation(Operation):

    def _parser(self, main_parser):
        main_parser.add_argument('env_file', help=".env file path", type=str)
        return main_parser

    def _before_start(self):
        super(DotEnvOperation, self)._before_start()
        load_dotenv(self.args.env_file)
        return True

    def _on_running(self):
        super(DotEnvOperation, self)._on_running()
        return True

    def _on_error(self, exception):
        super(DotEnvOperation, self)._on_error(exception)
        return True

    def _on_completed(self):
        super(DotEnvOperation, self)._on_completed()
        return True




from uperations.operation import Operation
import shutil
import os
from termcolor import cprint

class MakeOperationType(Operation):

    @staticmethod
    def name():
        return "make_operationtype"

    @staticmethod
    def description():
        return "Create a new operation type"

    def _parser(self, main_parser):
        main_parser.add_argument('type', help="Name of the operation type")
        return

    def _run(self):

        #Retrieve the library name and path
        operation_type = self.args.type

        template_path = os.path.join(os.path.dirname(os.path.relpath(__file__)),'resources','template')
        new_init_path = os.path.join('operation_types',operation_type,'__init__.py')

        shutil.copytree(template_path, os.path.join('operation_types', operation_type))
        with open(new_init_path,'r') as f:
            new_text = f.read().replace('OPERATIONTYPE', MakeOperationType.to_camel_case(operation_type).capitalize())

        with open(new_init_path,'w') as f:
            f.write(new_text)

        cprint("Operation type successfully created under: %s" % new_init_path,'green')
        return True


    @staticmethod
    def to_camel_case(snake_str):
        components = snake_str.split('_')
        return components[0] + ''.join(x.title() for x in components[1:])


from distutils.core import setup
from setuptools import find_packages

setup(
    name='LIBRARYNAME',
    version=None,
    packages=find_packages(),
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=open('README.rst').read(),
    author=None,
    author_email=None,
    keywords='workflow tool',
    url=None
)


import logging
import yaml
from jsonschema import Draft4Validator
from uperations.operation import Operation
from argparse import FileType

from abc import abstractmethod


class YmlConfigOperation(Operation):

    @property
    @abstractmethod
    def _config_schema(self):
        raise NotImplementedError("This operation needs a _config_schema function: "+self.__class__.__name__)

    def _parser(self, main_parser):
        main_parser.add_argument('config', help="Yaml Configuration file", type=FileType('r'))
        return main_parser

    def _before_start(self):
        super(YmlConfigOperation, self)._before_start()
        self.__load_config(self.args.config)
        self.__validate_config()
        return True

    def __validate_config(self):
        v = Draft4Validator(self._config_schema())
        errors = sorted(v.iter_errors(self.config), key=lambda e: e.path)
        if len(errors) > 0:
            for error in errors:
                logging.error("Confguration file: "+error.message)
        return

    def __load_config(self, yml_file_p):
        """
        Transform an opened yaml file in json

        Args:
            yml_file_p:  A pointer to a file opened in read mode

        Return:
            dict: The yaml file in dict format
        """
        self.config = yaml.load(yml_file_p)
        return




from uperations.library import Library
from .operations.make_library import Makelibrary
from .operations.make_operation import MakeOperation
from .operations.make_operationtype import MakeOperationType
from .operations.list_operation import Listoperation
from .operations.yaml_to_json import Yamltojson
from .operations.publish import Publish
from .operations.command import Command
from .operations.dockerfile import Dockerfile
from .operations.helloworld import Helloworld
from .operations.hello_name import HelloName
from .operations.make_observer import MakeObserver

class Base(Library):

    @staticmethod
    def name():
        return "base"

    @staticmethod
    def description():
        return "General operations"

    def _init_operations(self):
        self._operations = {
            'hello:world': Helloworld(self),
            'make:library': Makelibrary(self),
            'make:operation': MakeOperation(self),
            'make:operation:type': MakeOperationType(self),
            'make:observer': MakeObserver(self),
            'list:operations': Listoperation(self),
            'yaml_to_json': Yamltojson(self),
            'publish': Publish(self),
            'command': Command(self),
            'dockerfile': Dockerfile(self),
            'hello': HelloName(self)
        }
        return

    def operations(self):
        return self._operations



from uperations.operation import Operation
import shutil
import os

class Dockerfile(Operation):

    @staticmethod
    def name():
        return "Dockerfile"

    @staticmethod
    def description():
        return "Create a docker file at the root of the directory."

    def _parser(self, main_parser):
        main_parser.add_argument('--force', dest='force', help="Overwritte the Dockerfile if exists", action='store_true')
        return

    def _run(self):
        if os.path.isfile(os.path.join(os.getcwd(),"Dockerfile")):
            if not self.args.force:
                print("Dockerfile already exists. Use --force to overwrite: %s" % (os.path.join(os.getcwd(),'Dockerfile')))
                return

        shutil.copy(self.resource_file('Dockerfile'),os.path.join(os.getcwd(),"Dockerfile"))


    def _id_publishable(self):
        return False


import logging
from jsonschema import Draft4Validator
from uperations.operation import Operation
from argparse import FileType
from abc import abstractmethod
import json


class JsonOperation(Operation):

    @property
    @abstractmethod
    def _config_schema(self):
        raise NotImplementedError("This operation needs a _config_schema function: "+self.__class__.__name__)

    def _parser(self, main_parser):
        main_parser.add_argument('json', help="Json input file", type=FileType('r'))
        return main_parser

    def _before_start(self):
        super(JsonOperation, self)._before_start()
        self.json = json.load(self.args.json)
        return True

    def __validate_json(self):
        v = Draft4Validator(self._config_schema())
        errors = sorted(v.iter_errors(self.json), key=lambda e: e.path)
        if len(errors) > 0:
            for error in errors:
                logging.error("Json file: "+error.message)
        return



from ...operation_types.yml_config_operation import YmlConfigOperation
import json

class Yamltojson(YmlConfigOperation):

    @staticmethod
    def name():
        return "Yamltojson"

    @staticmethod
    def description():
        return "Display a yaml file as json"

    def _config_schema(self):
        return {}

    def _parser(self, main_parser):
        return

    def _run(self):
        print(json.dumps(self.config))
        return True



from uperations.operation_observer import OperationObserver

class OBSERVER_NAME(OperationObserver):

    def _before_start(self, operation, arg=None):
        return

    def _on_running(self, operation, arg=None):
        return

    def _on_completed(self, operation, arg=None):
        return

    def _on_error(self, error, operation, arg=None):
        return



from uperations.operation import Operation
from ..utils import create_observer
from uperations.kernel import Kernel

class MakeObserver(Operation):

    @staticmethod
    def name():
        return "make_observer"

    @staticmethod
    def description():
        return "Create an observer"

    def _parser(self, main_parser):
        main_parser.add_argument('library', help="Library name")
        main_parser.add_argument('observer', help="Observer name - No special characters")
        return

    def _run(self):
        create_observer(Kernel.get_instance().find_library(self.args.library), self.args.observer)
        return




from uperations.operation import Operation
from termcolor import colored
import tabulate
from uperations.kernel import Kernel


class Listoperation(Operation):

    @staticmethod
    def name():
        return "list_operation"

    @staticmethod
    def description():
        return "List all operations"

    def _parser(self, main_parser):
        main_parser.add_argument('-l', '--library', dest='library', required=False)
        return

    def _run(self):
        headers = ['Library', 'Command', 'Operation', 'Description']
        data = []
        for library, library_class in Kernel.get_instance().get_libraries().items():
            for operation, operation_class in library_class.operations().items():
                data.append([colored(library,'blue'), colored(operation, 'green'), operation_class.name(), operation_class.description()[:50]])
        print(tabulate.tabulate(data,headers))
        return True




from uperations.operation import Operation

class NEWOPERATION(Operation):

    @staticmethod
    def name():
        return "NEWOPERATION"

    @staticmethod
    def description():
        return "NEWOPERATION has not been documented yet."

    def _parser(self, main_parser):
        #main_parser.add_argument('first_argument', help="Argparse argument example")
        return

    def _run(self):
        raise NotImplementedError("NEWOPERATION not implemented yet.")



from uperations.library import Library

class LIBRARYNAME(Library):

    @staticmethod
    def name():
        return "LIBRARYNAME"

    @staticmethod
    def description():
        return "Not description provided"

    def operations(self):
        return {
        }




from uperations.operation import Operation
from ..utils import operation_create
from uperations.kernel import Kernel

class MakeOperation(Operation):

    @staticmethod
    def name():
        return "make_operation"

    @staticmethod
    def description():
        return "Create a new operation"

    def _parser(self, main_parser):
        main_parser.add_argument('library', help="Name of the library to add the operation")
        main_parser.add_argument('operation', help="Name of the operation")
        return

    def _run(self):
        operation_create(Kernel.get_instance().find_library(self.args.library),self.args.library,self.args.operation)
        return


from setuptools import setup, find_packages
import os

setup(
    name='uperations-base',
    version='0.0.12',
    packages=find_packages(),
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=open(os.path.join(os.path.dirname(os.path.realpath(__file__)),'README.rst')).read(),
    author='Brice Aminou',
    author_email='brice.aminou@gmail.com',
    keywords='workflow tool',
    url='https://github.com/baminou/Uperations-Base',
    package_data={'': ['operations/dockerfile/resources/Dockerfile']},
    install_requires=['jsonschema==2.6.0','uperations_kernel==0.0.6','tabulate==0.8.2','termcolor==1.1.0',
                      'setuptools==40.0.0','PyContracts==1.8.3','python-dotenv==0.9.1','PyYAML==3.13']
)